/*
 * dataset.h
 *
 * Copyright (C) 2001-2002 giFT project (gift.sourceforge.net)
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 */

#ifndef __DATASET_H
#define __DATASET_H

/*****************************************************************************/

#include "list.h"

/*****************************************************************************/

#ifdef __cplusplus
extern "C"
{
#endif /* __reallycrappylanguage */

/*****************************************************************************/

typedef enum
{
	DATASET_DEFAULT = 0,
	DATASET_LIST,
	DATASET_HASH
} DatasetType;

typedef struct _dataset
{
	DatasetType type;

	union
	{
		struct _hash_table *hash;
		List *list;
	} tdata;
} Dataset;

typedef struct _dataset_node
{
	void  *key;
	size_t key_len;
	void  *value;

	union
	{
		struct _dataset_node *hash_next;
		List *list_link;
	} tdata;
} DatasetNode;

typedef int (*DatasetForeach) (Dataset *d, DatasetNode *node, void *udata);

#define DATASET_FOREACH(func) ((DatasetForeach)func)

/*****************************************************************************/

typedef unsigned long (*HashFunc) (Dataset *d, void *key, size_t key_len);
typedef int (*HashCmpFunc) (Dataset *d, DatasetNode *node,
                            void *key, size_t key_len);

struct _hash_table
{
	size_t        size;
	unsigned long items;
	unsigned char frozen;

	DatasetNode **nodes;

	HashFunc    hash;
	HashCmpFunc cmp;
};

/*****************************************************************************/

Dataset *dataset_new (DatasetType type);
void dataset_clear (Dataset *d);
void dataset_clear_free (Dataset *d);

void dataset_insert (Dataset **d, void *key, size_t key_len, void *value);
void dataset_remove (Dataset *d, void *key, size_t key_len);
void *dataset_lookup (Dataset *d, void *key, size_t key_len);
DatasetNode *dataset_lookup_node (Dataset *d, void *key, size_t key_len);
void dataset_foreach (Dataset *d, DatasetForeach func, void *udata);
void dataset_foreach_ex (Dataset *d, DatasetForeach func, void *udata);
DatasetNode *dataset_find_node (Dataset *d, DatasetForeach func, void *udata);
void *dataset_find (Dataset *d, DatasetForeach func, void *udata);

unsigned long dataset_length (Dataset *d);
List *dataset_flatten (Dataset *d);

/*****************************************************************************/

#ifdef __cplusplus
}
#endif /* __reallycrappylanguage */

/*****************************************************************************/

#endif /* __DATASET_H */
