/*
 * interface.h
 *
 * Copyright (C) 2001-2002 giFT project (gift.sourceforge.net)
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 */

#ifndef __INTERFACE_H
#define __INTERFACE_H

/*****************************************************************************/

/**
 * @file interface.h
 *
 * @brief Interface protocol helpers
 *
 * Functions used to both read and write giFT's XML-like interface protocol.
 * These routines provide adequate enough abstraction to arbitrarily change
 * the protocol and still allow giFT and it's user interfaces to communicate
 * effectively.
 */

/*****************************************************************************/

#ifdef __cplusplus
extern "C"
{
#endif /* __reallycrappylanguage */

/*****************************************************************************/

char     *interface_construct  (int *len, Tree *packet);
void      interface_send       (Connection *c, Tree *packet);
void      interface_close      (Connection *c);
Tree     *interface_parse      (char *packet_str);
Tree     *interface_dup        (Tree *packet);
void      interface_free       (Tree *packet);
char     *interface_lookup     (Tree *packet, char *key);
char     *interface_lookup_mod (Tree *packet, char *key);
void      interface_set        (Tree **packet, char *key, char *mod,
                                char *value);
TreeNode *interface_append     (Tree **packet, TreeNode *parent,
                                char *key, char *mod, char *value);
void      interface_foreach    (Tree *packet, char *key, HashFunc func,
                                void *udata);

/*****************************************************************************/

#ifdef __cplusplus
}
#endif /* __reallycrappylanguage */

/*****************************************************************************/

#endif /* __INTERFACE_H */
